/*Copyright (C) 2009  Annette Bieniusa, Arie Middelkoop.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package vacation;

import dstm.Transactional;

@Transactional
public class Reservation {
	
	public enum ReservationType {CAR, FLIGHT, ROOM};
	
	//private int id;
	private int price;
	
	private int numUsed;
	private int numFree;
	private int numTotal;
	
	public Reservation(int id, int num, int price) {
		//this.id = id;
		this.numUsed = 0;
		this.numFree = num;
		this.numTotal = num;
		this.price = price;
	}

	public int getNumTotal(){
		return this.numTotal;
	}

	boolean addToTotal (int num){
		if (this.numFree + num < 0){
			return false; 
			}
		this.numFree += num;
		this.numTotal += num;
		
		//in original here is a consistency check!
		return true;
	}
	
	public boolean make() {
		if (this.numFree < 1) {
			return false;
		}
		this.numUsed ++;
		this.numFree --;
		return true;
	}

	 public void updatePrice(int price) {
		if (price >= 0) {
			this.price = price;
		}
	}

	public int getNumUsed() {
		return this.numUsed;
	}

	public boolean cancel() {
		if (this.numUsed < 1) {
			return false;
		}
		this.numUsed--;
		this.numFree++;
		return true;
	}

	public int getNumFree() {
		return this.numFree;
	}

	
	public int getPrice() {
		return this.price;
	}
	
	
	public boolean checkReservation(){
		return  (this.numUsed >= 0) & (this.numFree >= 0)  & 
		        (this.numTotal >= 0) & (this.price >= 0) &
				(this.numUsed + this.numFree == this.numTotal); 
	}
}
