/*Copyright (C) 2009  Annette Bieniusa, Arie Middelkoop.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package vacation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Random;

import vacation.Manager.Task;
/**
 * @author bieniusa
 *
 */
public class Vacation {
	static int PARAM_CLIENTS      = 4;
	static int PARAM_NUMBER       = 4;
	static int PARAM_QUERIES      = 60;
	static int PARAM_RELATIONS    = 128;
	static int PARAM_TRANSACTIONS = 1 << 10;
	static int PARAM_USER         = 60;
				
	static Collection<Client> initializeClients(Manager manager){
		System.out.print("Initializing clients...");
		
		Collection<Client> clients = new ArrayList<Client>(PARAM_CLIENTS);
		
		int numTransactionPerClient = (int) (.5 + PARAM_TRANSACTIONS / (double) PARAM_CLIENTS);
		int queryRange = (int) (0.5 + PARAM_RELATIONS * PARAM_QUERIES / 100.0);
		
		for (int i = 0; i < PARAM_CLIENTS; i ++){
			clients.add(new Client(i,manager,numTransactionPerClient,PARAM_NUMBER,queryRange,PARAM_USER));
		}
		
		System.out.println("done.");
		
		System.out.println("    Transactions          = "+ PARAM_TRANSACTIONS);
		System.out.println("    Clients               = "+ PARAM_CLIENTS);
		System.out.println("    Transactions/Client   = "+ numTransactionPerClient);
		System.out.println("    Queries/Transaction   = "+ PARAM_NUMBER);
		System.out.println("    Relations             = "+ PARAM_RELATIONS);
		System.out.println("    Query percentage      = "+ PARAM_QUERIES);
		System.out.println("    Query range           = "+ queryRange);
		System.out.println("    Percent user          = "+ PARAM_USER);
		
		return clients;
	}
	
	static Manager initializeManager(){
		
		System.out.print("Initializing manager...");
		Manager manager = Manager.init();
		Random randomgen = new Random(); 
		
		int[] ids = new int[PARAM_RELATIONS];
		for(int i = 0; i < PARAM_RELATIONS; i++){
			ids[i] = i + 1;
		}
		
		for (Task t : Task.values()) {
		/* Shuffle ids */
			for (int i = 0; i < PARAM_RELATIONS; i ++){
				int x = randomgen.nextInt(PARAM_RELATIONS);
				int y = randomgen.nextInt(PARAM_RELATIONS);
				int tmp = ids[x];
				ids[x] = ids[y];
				ids[y] = tmp;
			}
		
			for (int i = 0; i < PARAM_RELATIONS; i ++){
				int id = ids[i];
				int num = 100 * (randomgen.nextInt(5) + 1);
				int price = 10 * randomgen.nextInt(5) + 50;
				manager.addTask(t,id,num,price);
			}
		}	
		
		System.out.println("done."); 		
		return manager;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
	   
		Manager manager = initializeManager();
		Collection<Client> clients = initializeClients(manager);
		
		//initialize the STM system
		long startTime = System.currentTimeMillis();
		System.out.print("Running clients...");
		
		for (Client c : clients) {
			c.start();
		}
		
		for (Client c : clients) {
			try {
				c.join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		
		System.out.println("done.");
		long endTime = System.currentTimeMillis();
		long diffTime = (endTime - startTime) % 1000;
		System.out.println("Time (in ms)= " + diffTime);
		
		//shutdown STM system
		
		//checkTables(manager);
		
	}
	/*
	private static void checkTables(Manager manager) {
		System.out.print("Checking tables...");
		
		/* Check for unique customer IDs */
		/* this is completely stupid.... */
		/* when using a hash table, there will always be at maximum one entry per hash (=id) */
		/*
		int queryRange = (int) (0.5 + PARAM_RELATIONS * PARAM_QUERIES / 100.0);
		Map<Integer, Customer> customerTable = manager.getCustomerTable();
		for (int i = 0; i < queryRange; i ++) {
			if (customerTable.get(i) != null) {
				customerTable.remove(i);
				if (customerTable.get( i) != null){
					System.out.println("Found inconsistency in customer table!");
				}
			}
		}
		
		
		
		for (ReservationType t : ReservationType.values()){
			RBTree<Integer, Reservation> table = manager.getAssociatedReservationTable(t);
			for (int i = 0; i < PARAM_RELATIONS; i++){
				boolean consistent = true;
				Reservation res = table.get(i);
				if (res!= null){
					consistent = res.checkReservation();
					/* this is also stupid!
					table.remove(i);
					consistent = consistent && (table.get(i) == null);
					
					}
				if (!consistent){
				System.out.println("Found inconsistency in reservation table!");		
				}
			}
		}
		
		
		System.out.println("done.");
	}
	*/
}
